<?php

/**
 * @file
 * Administrative functions for MaPS Import module.
 */

use Drupal\maps_import\Profile\Profile;
use Drupal\maps_suite\Log\Logger;

/**
 * Form builder; manual import form.
 *
 * @see maps_import_import_form_validate()
 * @see maps_import_import_form_submit()
 * @ingroup forms
 */
function maps_import_import_form($form, &$form_state, Profile $profile) {
  drupal_set_title(t('Manual import for profile @profile', array('@profile' => $profile->getTitle())));

  if (variable_get('maps_import_lock', 0) == 1) {
    drupal_set_message(t('There is already an operation under processing. Would you like to <a href="@url"> break the lock</a> ?', array('@url' => '/admin/maps-suite/lock/break')), 'warning');
  }

  $form['#profile'] = $profile;
  $form['info'] = array(
    '#markup' => t('All enabled profiles are usually processed through Drush scripts. However you may want to process some import operation manually, for either enabled or disabled profiles.'));

  $operations = maps_import_get_import_operations($profile);

  $form['operation'] = array(
    '#type' => 'radios',
    '#title' => t('Operation to perform'),
    '#options' => maps_suite_reduce_array($operations, 'title'),
    '#required' => TRUE,
  );

  foreach ($operations as $type => $operation) {
    $form['description'][$type] = array(
      '#type' => 'item',
      '#markup' => '<div class="description">' . $operation['description'] . '</div>',
      '#states' => array(
        'visible' => array(
          ':input[name="operation"]' => array('value' => $type),
        ),
      ),
    );
  }

  // Do not allow object operations if configuration is empty.
  if (!$profile->getConfiguration()) {
    $form['operation']['#options'] = array_intersect_key($form['operation']['#options'], array('configuration_fetch' => TRUE));
    drupal_set_message(t('The only available operation is configuration import, while configuration is empty.'), 'warning');
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Process'),
  );

  return $form;
}

/**
 * Form validation handler; .
 *
 * @see maps_import_import_form()
 * @see maps_import_import_form_submit()
 */
function maps_import_import_form_validate($form, &$form_state) {
  // Force maintenance mode for large operations.
  $operation = maps_import_get_import_operations($form['#profile'], $form_state['values']['operation']);

  if (!empty($operation['maintenance']) && !variable_get('maintenance_mode', 0)) {
    form_set_error('operation', t('Operation %title can only be run in maintenance mode. Please use this !link to modify maintenance state.', array(
      '%title' => $operation['title'],
      '!link' => l(t('link'),
      'admin/config/development/maintenance', array('query' => drupal_get_destination())))));
  }
}

/**
 * Form submission handler; manual import form.
 *
 * @see maps_import_import_form()
 * @see maps_import_import_form_validate()
 */
function maps_import_import_form_submit($form, &$form_state) {
  if ($operation = maps_import_get_import_operations($form['#profile'], $form_state['values']['operation'])) {
    $fetcher = new $operation['class']($form['#profile']);

    $batch = array(
      'operations' => $fetcher->batchOperations(),
      'title' => t('Processing %op operation for the profile %title', array('%op' => $operation['title'], '%title' => $form['#profile']->getTitle())),
      'file' => drupal_get_path('module', 'maps_import') . '/admin/maps_import.admin.inc',
      'finished' => 'maps_import_import_finished',
    );

    $test_info = $GLOBALS['drupal_test_info'];
    if (!empty($test_info['test_run_id'])) {
      $fetcher->process();
    }
    else {
      if (variable_get('maps_import_lock', 0) == 1) {
        drupal_set_message(t('There is already operations under processing'));
        return;
      }

      if (!empty($batch['operations'])) {
        // Set the lock.
        variable_set('maps_import_lock', 1);
        batch_set($batch);
      }
      else {
        drupal_set_message(t('There is nothing to do.'));
      }
    }
  }
}

/**
 * Batch "finished" callback; manual import.
 */
function maps_import_import_finished($success, $results, $operations) {
  // Remove the lock.
  variable_set('maps_import_lock', 0);

  $message = array();
  $message_type = 'status';

  if (!empty($results['log']) && file_exists($results['log'])) {
    $message['log_link'] = t('The log file is available !log_url.', array('!log_url' => l(t('here'), $results['log'])));
  }

  if (isset($results['operation'])) {
    module_invoke_all('maps_import_operation_finished', $results['operation'], $message);
  }

  if ($success) {
    // Check if something went wrong.
    if (isset($results['process']) && FALSE === $results['process']) {
      array_unshift($message, t('The operation %name failed.', array('%name' => $results['operation'])));
      $message_type = 'error';
    }
    else {
      array_unshift($message, t('The operation %name has been executed with success.', array('%name' => $results['operation'])));

      // Total operations.
      if (!empty($results['total'])) {
        $message[] = t('%count operations:', array('%count' => $results['total']));
      }

      // Details for updated / imported elements.
      if (!empty($results['indexes'])) {
        foreach ($results['indexes'] as $element => $index) {
          if (!empty($index['insert'])) {
            $message[] = t('%count %element have been added', array('%count' => $index['insert'], '%element' => $element));
          }

          if (!empty($index['update'])) {
            $message[] = t('%count %element have been updated', array('%count' => $index['update'], '%element' => $element));
          }
        }
      }
      if (!empty($results['indexes']['fetch'])) {
        $message[] = t('Succesfully fetched data from %count file from MaPS System®.', array('%count' => $results['indexes']['fetch']));
      }
    }
  }
  // An error occurred.
  else {
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    list($class, $method) = $error_operation[0];
    $message_type = 'error';
    $message[] = t('An error occurred while processing @operation with arguments :<br />!args<hr />Error message is: !message',
      array(
        '@operation' => get_class($class) . '::' . $method,
        '!args' => maps_suite_highlight_php($error_operation[1], TRUE),
        '!message' => isset($results['error']) ? $results['error'] : t('not specified')));

  }

  drupal_set_message(theme('item_list', array('items' => $message)), $message_type);
}
